/*
 * main.c
 *
 *  Created on: Apr 26, 2018
 *      Author: Pramanujam
 */






#include <stdint.h>
#include <stdbool.h>
#include "inc/hw_types.h"
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "driverlib/gpio.h"
#include "driverlib/i2c.h"
#include "driverlib/interrupt.h"
#include "driverlib/pin_map.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "utils/random.h"
#include "utils/uartstdio.h"



#define SLAVE_ADDRESS 0x68 // Address appears from bits 7:1 D0 shifted 1 place to right is 68
#define NUM_OF_I2CBYTES   2


/*****************************************************************************
//
// Global variables
//
//*****************************************************************************/
uint32_t pui32DataRx[NUM_OF_I2CBYTES];

void I2C1IntHandler(void)
{

}

//*****************************************************************************
//
// This function sets up UART0 to be used for a console to display information
// as the example is running.
//
//*****************************************************************************
void
InitConsole(void)
{
    //
    // Enable GPIO port A which is used for UART0 pins.
    // TODO: change this to whichever GPIO port you are using.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);

    //
    // Configure the pin muxing for UART0 functions on port A0 and A1.
    // This step is not necessary if your part does not support pin muxing.
    // TODO: change this to select the port/pin you are using.
    //
    GPIOPinConfigure(GPIO_PA0_U0RX);
    GPIOPinConfigure(GPIO_PA1_U0TX);

    //
    // Enable UART0 so that we can configure the clock.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);

    //
    // Use the internal 16MHz oscillator as the UART clock source.
    //
    UARTClockSourceSet(UART0_BASE, UART_CLOCK_PIOSC);

    //
    // Select the alternate (UART) function for these pins.
    // TODO: change this to select the port/pin you are using.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Initialize the UART for console I/O.
    //
    UARTStdioConfig(0, 115200, 16000000);
}


//*****************************************************************************
//
// Main Program to Configure and Use the I2C Master I2C1
//
//*****************************************************************************
int
main(void)
{
    uint32_t ui32SysClock;


    bool     bError;

    //
        // Setup System Clock for 120MHz
        //
        ui32SysClock = SysCtlClockFreqSet((SYSCTL_OSC_MAIN | SYSCTL_USE_PLL | SYSCTL_XTAL_25MHZ |
                            SYSCTL_CFG_VCO_480), 120000000);


    // Set up the serial console to use for displaying messages.  This is
    // just for this example program and is not needed for EPI operation.
    //
    InitConsole();

    //
    // Display the setup on the console.
    //
    UARTprintf("\033[2J\033[H");
    UARTprintf("\r\nExample Code for Tiva I2C1 Master with LTC4151 as slave\n");


    // Enable GPIO for Configuring the I2C Interface Pins
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOG);

        // Wait for the Peripheral to be ready for programming
        //
        while(!SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOG));

        //
        // Configure Pins for I2C1 Master Interface
        //
        GPIOPinConfigure(GPIO_PG0_I2C1SCL);
        GPIOPinConfigure(GPIO_PG1_I2C1SDA);

        GPIOPinTypeI2CSCL(GPIO_PORTG_BASE, GPIO_PIN_1);
        GPIOPinTypeI2C(GPIO_PORTG_BASE, GPIO_PIN_0);

        //
            // Stop the Clock, Reset and Enable I2C Module
            // in Master Function
            //
            SysCtlPeripheralDisable(SYSCTL_PERIPH_I2C1);
            SysCtlPeripheralReset(SYSCTL_PERIPH_I2C1);
            SysCtlPeripheralEnable(SYSCTL_PERIPH_I2C1);


            //
            // Wait for the Peripheral to be ready for programming
            //
            while(!SysCtlPeripheralReady(SYSCTL_PERIPH_I2C1));

            // Initialize and Configure the Master Module
                //
                I2CMasterInitExpClk(I2C1_BASE, ui32SysClock, true);

                // --------------------------To Write before read-----------------------

                      I2CMasterSlaveAddrSet (I2C1_BASE, SLAVE_ADDRESS, false); //set slave address and initiate with write

                      I2CMasterControl (I2C1_BASE, I2C_MASTER_CMD_SINGLE_SEND); //Send STOP bit

                      I2CMasterDataPut (I2C1_BASE, 0x00); //Put 0x00 -- send address of register to be read from SLAVE



                      while (!(I2CMasterBusy(I2C1_BASE))); //Wait till end of transaction
          //            while ((I2CMasterBusy(I2C1_BASE))); //Wait till end of transaction

                      I2CMasterSlaveAddrSet (I2C1_BASE, SLAVE_ADDRESS, true); //set slave address and initiate with read

                      I2CMasterControl (I2C1_BASE, I2C_MASTER_CMD_BURST_RECEIVE_START); //Read the 1st Byte

                      pui32DataRx[0] = I2CMasterDataGet(I2C1_BASE);

                      while (!(I2CMasterBusy(I2C1_BASE))); //Wait till end of transaction
    //                  while ((I2CMasterBusy(I2C1_BASE))); //Wait till end of transaction


                      I2CMasterControl (I2C1_BASE, I2C_MASTER_CMD_BURST_RECEIVE_FINISH); //Read the 2nd Byte

                      pui32DataRx[1] = I2CMasterDataGet(I2C1_BASE);

                      while (!(I2CMasterBusy(I2C1_BASE))); //Wait till end of transaction
     //                 while ((I2CMasterBusy(I2C1_BASE))); //Wait till end of transaction


                      UARTprintf("Receiving data %d  and data 2 %d from external Slave...\n\n",pui32DataRx[0],pui32DataRx[1]);

                      while (1)
                      {
                      }
 } //end main
